

using UnityEngine;

/// <summary>
/// Represents a single image channel (e.g., Red, Green, Blue, Alpha) as a 1D byte array,
/// along with the original image dimensions.
/// </summary>
public struct SplitChannelData
{
    /// <summary>
    /// The 8-bit intensity values for a single image channel, stored in row-major order.
    /// </summary>
    public byte[] ChannelData;

    /// <summary>
    /// The width and height of the channel data (width, height).
    /// </summary>
    public (int width, int height) Dimensions;

    /// <summary>
    /// Initializes a new instance of the <see cref="SplitChannelData"/> struct
    /// using channel data and dimensions.
    /// </summary>
    /// <param name="channelData">The byte array representing the single channel values.</param>
    /// <param name="widthHeight">A tuple of (width, height) representing the dimensions.</param>
    public SplitChannelData(byte[] channelData, (int width, int height) widthHeight)
    {
        ChannelData = channelData;
        Dimensions = widthHeight;
    }

    /// <summary>
    /// Gets the width of the channel data.
    /// </summary>
    public int Width => Dimensions.width;

    /// <summary>
    /// Gets the height of the channel data.
    /// </summary>
    public int Height => Dimensions.height;

    /// <summary>
    /// Checks whether the channel data is valid (non-null and matches the expected size).
    /// </summary>
    public bool IsValid => ChannelData != null && ChannelData.Length == Width * Height;
}
